from _thread import start_new_thread
import queue
import requests
import xbmc
from helper import utils


class HTTP:
    def __init__(self, EmbyServer):
        self.session = None
        self.EmbyServer = EmbyServer
        self.Intros = []
        self.HeaderCache = {}
        self.AsyncCommandQueue = queue.Queue()

    def async_commands(self):
        xbmc.log("EMBY.emby.http: THREAD: --->[ async queue ]", 1) # LOGINFO
        CommandRetry = ()
        CommandRetryCounter = 0

        while True:
            if CommandRetry:
                Command = CommandRetry
                CommandRetry = ()
            else:
                Command = self.AsyncCommandQueue.get()

            try:
                if Command[0] == "POST":
                    Command[1]['timeout'] = (5, 5)
                    r = self.session.post(**Command[1])
                    r.close()
                elif Command[0] == "DELETE":
                    Command[1]['timeout'] = (5, 5)
                    r = self.session.delete(**Command[1])
                    r.close()
                elif Command[0] == "QUIT":
                    xbmc.log("EMBY.emby.http: Queue closed", 1) # LOGINFO
                    break

                CommandRetryCounter = 0
            except Exception as error:
                if CommandRetryCounter < 5:
                    CommandRetryCounter += 1
                    CommandRetry = Command
                    xbmc.log(f"EMBY.emby.http: Async_commands retry: {CommandRetryCounter}", 2) # LOGWARNING
                else:
                    CommandRetryCounter = 0
                    CommandRetry = ()
                    xbmc.log(f"EMBY.emby.http: Async_commands error: {error}", 2) # LOGWARNING

        xbmc.log("EMBY.emby.http: THREAD: ---<[ async queue ]", 1) # LOGINFO

    def stop_session(self):
        if not self.session:
            return

        LocalSession = self.session  # Use local var -> self.session must be set to "none" instantly -> self var is also used to detect open sessions
        self.session = None
        self.AsyncCommandQueue.put(("QUIT",))

        try:
            LocalSession.close()
        except Exception as error:
            xbmc.log(f"EMBY.emby.http: Session close error: {error}", 2) # LOGWARNING

        xbmc.log("EMBY.emby.http: Session close", 1) # LOGINFO

    # decide threaded or wait for response
    def request(self, data, ForceReceiveData, Binary, GetHeaders=False):
        RequestType = data.pop('type', "GET")

        if 'url' not in data:
            data['url'] = f"{self.EmbyServer.ServerData['ServerUrl']}/emby/{data.pop('handler', '')}"

        if 'headers' not in data:
            Header = {'Content-type': "application/json", 'Accept-Charset': "UTF-8,*", 'Accept-encoding': "gzip", 'User-Agent': f"{utils.addon_name}/{utils.addon_version}"}
        else:
            Header = data['headers']
            del data['headers']

        if 'Authorization' not in Header:
            auth = f"Emby Client={utils.addon_name},Device={utils.device_name},DeviceId={utils.device_id},Version={utils.addon_version}"

            if self.EmbyServer.ServerData['AccessToken'] and self.EmbyServer.ServerData['UserId']:
                Header.update({'Authorization': f"{auth},UserId={self.EmbyServer.ServerData['UserId']}", 'X-Emby-Token': self.EmbyServer.ServerData['AccessToken']})
            else:
                Header.update({'Authorization': auth})

        data['timeout'] = (30, 300)
        xbmc.log(f"EMBY.emby.http: [ http ] {data}", 0) # LOGDEBUG

        # Shutdown
        if utils.SystemShutdown:
            self.stop_session()
            return noData(Binary, GetHeaders)

        # start session
        if not self.session:
            self.HeaderCache = {}
            self.session = requests.Session()
            start_new_thread(self.async_commands, ())

        # Update session headers
        if Header != self.HeaderCache:
            self.HeaderCache = Header.copy()
            Header.update({'Connection': 'Keep-Alive'})
            self.session.headers = Header

        # http request
        try:
            if RequestType == "HEAD":
                r = self.session.head(**data)
                r.close()
                return r.status_code

            if RequestType == "GET":
                r = self.session.get(**data)
                r.close()

                if r.status_code == 200:
                    if Binary:
                        if GetHeaders:
                            return r.content, r.headers

                        return r.content

                    return r.json()

                if r.status_code == 401:
                    utils.Dialog.notification(heading=utils.addon_name, message=utils.Translate(33147))

                xbmc.log(f"EMBY.emby.http: [ Statuscode ] {r.status_code}", 3) # LOGERROR
                xbmc.log(f"EMBY.emby.http: [ Statuscode ] {data}", 0) # LOGDEBUG
                return noData(Binary, GetHeaders)
            if RequestType == "POST":
                if ForceReceiveData:
                    r = self.session.post(**data)
                    r.close()
                    return r.json()

                self.AsyncCommandQueue.put(("POST", data))
            elif RequestType == "DELETE":
                self.AsyncCommandQueue.put(("DELETE", data))

            return noData(Binary, GetHeaders)
        except requests.exceptions.SSLError:
            xbmc.log("EMBY.emby.http: [ SSL error ]", 3) # LOGERROR
            xbmc.log(f"EMBY.emby.http: [ SSL error ] {data}", 0) # LOGDEBUG
            utils.Dialog.notification(heading=utils.addon_name, message=utils.Translate(33428))
            self.stop_session()
            return noData(Binary, GetHeaders)
        except requests.exceptions.ConnectionError:
            xbmc.log("EMBY.emby.http: [ ServerUnreachable ]", 3) # LOGERROR
            xbmc.log(f"EMBY.emby.http: [ ServerUnreachable ] {data}", 0) # LOGDEBUG
            self.stop_session()
            self.EmbyServer.ServerUnreachable()
            return noData(Binary, GetHeaders)
        except requests.exceptions.ReadTimeout:
            xbmc.log("EMBY.emby.http: [ ServerTimeout ]", 3) # LOGERROR
            xbmc.log(f"EMBY.emby.http: [ ServerTimeout ] {data}", 0) # LOGDEBUG
            self.stop_session()
            return noData(Binary, GetHeaders)
        except Exception as error:
            xbmc.log(f"EMBY.emby.http: [ Unknown ] {error}", 3) # LOGERROR
            xbmc.log(f"EMBY.emby.http: [ Unknown ] {data} / {error}", 0) # LOGDEBUG
            self.stop_session()
            return noData(Binary, GetHeaders)

    def load_Trailers(self, EmbyId):
        ReceivedIntros = []
        self.Intros = []

        if utils.localTrailers:
            IntrosLocal = self.EmbyServer.API.get_local_trailers(EmbyId)

            for IntroLocal in IntrosLocal:
                ReceivedIntros.append(IntroLocal)

        if utils.Trailers:
            IntrosExternal = self.EmbyServer.API.get_intros(EmbyId)

            if 'Items' in IntrosExternal:
                for IntroExternal in IntrosExternal['Items']:
                    ReceivedIntros.append(IntroExternal)

            if ReceivedIntros:
                Index = 0

                for Index, Intro in enumerate(ReceivedIntros):
                    if self.verify_intros(Intro):
                        break

                for Intro in ReceivedIntros[Index + 1:]:
                    start_new_thread(self.verify_intros, (Intro,))

    def verify_intros(self, Intro):
        xbmc.log("EMBY.emby.http: THREAD: --->[ verify intros ]", 1) # LOGINFO

        if Intro['Path'].find("http") == -1: # Local Trailer
            Intro['Path'] = f"{self.EmbyServer.ServerData['ServerUrl']}/emby/videos/{Intro['Id']}/stream?static=true&api_key={self.EmbyServer.ServerData['AccessToken']}&DeviceId={utils.device_id}"
            self.Intros.append(Intro)
            xbmc.log("EMBY.emby.http: THREAD: ---<[ verify intros ] local trailer", 1) # LOGINFO
            return True

        try:
            r = requests.head(Intro['Path'], allow_redirects=True, timeout=2)
            r.close()

            if Intro['Path'] == r.url:
                self.Intros.append(Intro)
                xbmc.log("EMBY.emby.http: THREAD: ---<[ verify intros ] remote trailer", 1) # LOGINFO
                return True

            # filter URL redirections, mostly invalid links
            xbmc.log(f"EMBY.emby.http: Invalid Trailer Path (url compare): {Intro['Path']} / {r.url}", 3) # LOGERROR
        except Exception as Error:
            xbmc.log(f"EMBY.emby.http: Invalid Trailer Path: {Intro['Path']} / {Error}", 3) # LOGERROR

        xbmc.log("EMBY.emby.http: THREAD: ---<[ verify intros ] invalid", 1) # LOGINFO
        return False

def noData(Binary, GetHeaders):
    if Binary:
        if GetHeaders:
            return b"", {}

        return b""

    return {}
