import uuid
from urllib.parse import unquote_plus
import json
import xbmc
from database import dbio
from emby import listitem
from helper import utils, pluginmenu
from dialogs import skipintrocredits

PlaylistRemoveItem = -1
result = json.loads(xbmc.executeJSONRPC('{"jsonrpc": "2.0", "id": 1, "method": "Application.GetProperties", "params": {"properties": ["volume", "muted"]}}')).get('result', {})
Volume = result.get('volume', 0)
Muted = result.get('muted', False)
PlayingItem = {}
QueuedPlayingItem = {}
EmbyServerPlayback = None
PlayingVideoAudio = True
MultiselectionDone = False
PositionTrackerThread = False
TrailerPath = ""
playlistIndex = -1
SkipItem = False
KodiMediaType = ""
LibraryId = ""
PlayBackEnded = True
IntroStartPositionTicks = 0
IntroEndPositionTicks = 0
CreditsPositionTicks = 0
LiveStreamId = None
SkipIntroJumpDone = False
SkipCreditsJumpDone = False
SkipAVChange = False
SkipIntroDialog = skipintrocredits.SkipIntro("SkipIntroDialog.xml", *utils.CustomDialogParameters)
SkipIntroDialogEmbuary = skipintrocredits.SkipIntro("SkipIntroDialogEmbuary.xml", *utils.CustomDialogParameters)
SkipCreditsDialog = skipintrocredits.SkipIntro("SkipCreditsDialog.xml", *utils.CustomDialogParameters)
playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)

def Play():
    if not PlayBackEnded:
        stop_playback(True, False)

    xbmc.log("EMBY.hooks.player: [ onPlayBackStarted ]", 1) # LOGINFO

    if not utils.syncduringplayback:
        utils.SyncPause['playing'] = True

def AVChange():
    xbmc.log("EMBY.hooks.player: [ onAVChange ]", 1) # LOGINFO

    if SkipAVChange:
        xbmc.log("EMBY.hooks.player: skip onAVChange: SkipAVChange", 1) # LOGINFO
        globals()["SkipAVChange"] = False
        return

    if EmbyServerPlayback and "ItemId" in PlayingItem:
        if PlaylistRemoveItem != -1 or SkipItem:
            xbmc.log("EMBY.hooks.player: skip onAVChange", 0) # LOGDEBUG
            return

        xbmc.log("EMBY.hooks.player: onAVChange update progress", 1) # LOGINFO
        globals()["PlayingItem"].update({'RunTimeTicks': int(utils.XbmcPlayer.getTotalTime() * 10000000), 'PositionTicks': max(int(utils.XbmcPlayer.getTime() * 10000000), 0)})
        EmbyServerPlayback.API.session_progress(PlayingItem)

def AVStart(EventData):
    xbmc.log("EMBY.hooks.player: [ onAVStarted ]", 1) # LOGINFO
    close_SkipIntroDialog()
    close_SkipCreditsDialog()
    globals().update({"SkipIntroJumpDone": False, "SkipCreditsJumpDone": False})

    if not utils.syncduringplayback:
        utils.SyncPause['playing'] = True

    # Trailer from webserverice (addon mode)
    if SkipItem:
        return

    # 3D, ISO etc. content from webserverice (addon mode)
    if PlaylistRemoveItem != -1:
        xbmc.executeJSONRPC(f'{{"jsonrpc":"2.0", "method":"Playlist.Remove", "params":{{"playlistid":1, "position":{PlaylistRemoveItem}}}}}')
        globals()["PlaylistRemoveItem"] = -1

    # Native mode multiselection
    if MultiselectionDone:
        globals()["MultiselectionDone"] = False
        xbmc.executebuiltin('ActivateWindow(12005)')  # focus videoplayer
        return

    globals().update({"MediaType": "", "LibraryId": ""})
    EmbyId = None

    try:
        if utils.XbmcPlayer.isPlayingVideo():
            PlayerItem = utils.XbmcPlayer.getVideoInfoTag()
            FullPath = PlayerItem.getFilenameAndPath()
            FocusVideoScreen = True
        elif utils.XbmcPlayer.isPlayingAudio():
            PlayerItem = utils.XbmcPlayer.getMusicInfoTag()
            FullPath = PlayerItem.getURL()
            FocusVideoScreen = False

        globals()["PlayingVideoAudio"] = True
    except:
        # Bluray
        xbmc.log("EMBY.hooks.player: No path, probably bluray detected", 1) # LOGINFO
        FullPath = ""
        FocusVideoScreen = True
        globals()["PlayingVideoAudio"] = False

    EventData = json.loads(EventData)

    if not 'id' in EventData['item']:
        # Update player info for dynamic content (played via widget)
        for CacheList in list(pluginmenu.QueryCache.values()):
            for ListItemCache in CacheList[1]:
                if ListItemCache[0] == FullPath:
                    xbmc.log("EMBY.hooks.player: Update player info", 1) # LOGINFO
                    utils.XbmcPlayer.updateInfoTag(ListItemCache[1])
                    break

        pluginmenu.reset_querycache() # Clear Cache
    else:
        KodiId = EventData['item']['id']

    globals().update({"KodiMediaType": EventData['item']['type'], "LibraryId": ""})

    # Extract LibraryId from Path
    if FullPath and FullPath.startswith("http://127.0.0.1:57342"):
        Temp = FullPath.split("/")

        if len(Temp) > 5:
            globals()["LibraryId"] = Temp[4]

    # extract path for bluray or iso (native mode)
    if utils.useDirectPaths and not FullPath:
        PlayingFile = utils.XbmcPlayer.getPlayingFile()

        if PlayingFile.startswith("bluray://"):
            PlayingFile = unquote_plus(PlayingFile)
            PlayingFile = unquote_plus(PlayingFile)
            PlayingFile = PlayingFile.replace("bluray://", "")
            PlayingFile = PlayingFile.replace("udf://", "")
            PlayingFile = PlayingFile[:PlayingFile.find("//")]

            for server_id, EmbyServer in list(utils.EmbyServers.items()):
                globals()["EmbyServerPlayback"] = EmbyServer
                embydb = dbio.DBOpenRO(server_id, "onAVStarted")
                EmbyId = embydb.get_mediasource_EmbyID_by_path(PlayingFile)

                if EmbyId:
                    FullPath = PlayingFile

                dbio.DBCloseRO(server_id, "onAVStarted")

    # native mode
    if FullPath and not FullPath.startswith("http") and not FullPath.startswith("/emby_addon_mode/"):
        PlaySessionId = str(uuid.uuid4()).replace("-", "")
        MediasourceID = ""

        for server_id, EmbyServer in list(utils.EmbyServers.items()):
            globals()["EmbyServerPlayback"] = EmbyServer
            embydb = dbio.DBOpenRO(server_id, "onAVStarted")
            item = embydb.get_item_by_KodiId_KodiType(KodiId, KodiMediaType)

            if not item:
                dbio.DBCloseRO(server_id, "onAVStarted")
                continue

            EmbyId = item[0][0]

            # Cinnemamode
            if (utils.enableCinemaMovies and item[0][3] == "movie") or (utils.enableCinemaEpisodes and item[0][3] == "episode"):
                if TrailerPath != "START_MAIN_CONTENT":
                    if TrailerPath != utils.XbmcPlayer.getPlayingFile():  # If not currently playing a trailer, initiate new trailers
                        utils.XbmcPlayer.pause()  # Player Pause
                        EmbyServerPlayback.http.Intros = []
                        PlayTrailer = True

                        if utils.askCinema:
                            PlayTrailer = utils.Dialog.yesno(heading=utils.addon_name, message=utils.Translate(33016), autoclose=int(utils.autoclose) * 1000)

                        if PlayTrailer:
                            EmbyServerPlayback.http.load_Trailers(EmbyId)

                        if EmbyServerPlayback.http.Intros:
                            globals()["playlistIndex"] = playlist.getposition()
                            globals()["SkipAVChange"] = True
                            play_Trailer()
                            dbio.DBCloseRO(server_id, "onAVStarted")
                            return

                        globals()["TrailerPath"] = ""
                        utils.XbmcPlayer.pause()  # Player resume
                else:
                    globals()["TrailerPath"] = ""

            # Multiversion
            MediaSources = embydb.get_mediasource(EmbyId)
            dbio.DBCloseRO(server_id, "onAVStarted")

            if MediaSources: # video
                MediasourceID = MediaSources[0][2]

                if len(MediaSources) > 1:
                    utils.XbmcPlayer.pause()  # Player Pause
                    Selection = []

                    for MediaSource in MediaSources:
                        Selection.append(f"{MediaSource[4]} - {utils.SizeToText(float(MediaSource[5]))} - {MediaSource[3]}")

                    MediaIndex = utils.Dialog.select(utils.Translate(33453), Selection)

                    if MediaIndex == -1:
                        Cancel()
                        return

                    if MediaIndex == 0:
                        utils.XbmcPlayer.pause()  # Player Resume
                    else:
                        globals()["MultiselectionDone"] = True
                        Path = MediaSources[MediaIndex][3]

                        if Path.startswith('\\\\'):
                            Path = Path.replace('\\\\', "smb://", 1).replace('\\\\', "\\").replace('\\', "/")

                        ListItem = load_KodiItem("onAVStarted", KodiId, KodiMediaType, Path)

                        if not ListItem:
                            return

                        globals()["playlistIndex"] = playlist.getposition()
                        playlist.add(Path, ListItem, playlistIndex + 1)
                        MediasourceID = MediaSources[MediaIndex][2]
                        utils.XbmcPlayer.playnext()
                        xbmc.executeJSONRPC(f'{{"jsonrpc":"2.0", "method":"Playlist.Remove", "params":{{"playlistid":1, "position":{playlistIndex}}}}}')

                    break
            else:
                MediasourceID = ""

        if EmbyId:
            queuePlayingItem(EmbyId, MediasourceID, PlaySessionId, item[0][12], item[0][13], item[0][14])

    globals().update({"PlayBackEnded": False, "PlayingItem": QueuedPlayingItem, "QueuedPlayingItem": {}})

    if EmbyServerPlayback and 'ItemId' in PlayingItem:
        if FocusVideoScreen:
            xbmc.executebuiltin('ActivateWindow(12005)')  # focus videoplayer

        utils.ItemSkipUpdate += [PlayingItem['ItemId'], PlayingItem['ItemId'], PlayingItem['ItemId']] # triple add -> for Emby (2 times incoming msg) and once for Kodi database incoming msg
        globals()["PlayingItem"].update({'RunTimeTicks': int(utils.XbmcPlayer.getTotalTime() * 10000000), 'PositionTicks': max(int(utils.XbmcPlayer.getTime() * 10000000), 0)})
        EmbyServerPlayback.API.session_playing(PlayingItem)
        xbmc.log(f"EMBY.hooks.player: ItemSkipUpdate: {utils.ItemSkipUpdate}", 0) # LOGDEBUG

        if not PositionTrackerThread:
            globals()["PositionTrackerThread"] = True
            PositionTracker()

def Seek(EventData):
    xbmc.log("EMBY.hooks.player: [ onPlayBackSeek ]", 1) # LOGINFO
    globals()["SkipAVChange"] = True

    if not EmbyServerPlayback or PlaylistRemoveItem != -1 or 'ItemId' not in PlayingItem and not PlayingVideoAudio:
        return

    EventData = json.loads(EventData)
    SeekPosition = (EventData['player']['time']['hours'] * 3600000 + EventData['player']['time']['minutes'] * 60000 + EventData['player']['time']['seconds'] * 1000 + EventData['player']['time']['milliseconds']) * 10000
    globals()["PlayingItem"]['PositionTicks'] = SeekPosition

    for SessionId in utils.WatchTogetherParticipants:
        if SessionId != EmbyServerPlayback.EmbySession[0]['Id']:
            EmbyServerPlayback.API.send_seek(SessionId, PlayingItem['PositionTicks'])

    EmbyServerPlayback.API.session_progress(PlayingItem)

def Pause():
    xbmc.log("EMBY.hooks.player: [ onPlayBackPaused ]", 1) # LOGINFO

    if not EmbyServerPlayback or 'ItemId' not in PlayingItem:
        return

    PositionTicks = max(int(utils.XbmcPlayer.getTime() * 10000000), 0)
    globals()["PlayingItem"].update({'PositionTicks': PositionTicks, 'IsPaused': True})

    if utils.WatchTogetherInProgress:
        for SessionId in utils.WatchTogetherParticipants:
            if SessionId != EmbyServerPlayback.EmbySession[0]['Id']:
                EmbyServerPlayback.API.send_pause(SessionId)
                EmbyServerPlayback.API.send_seek(SessionId, PlayingItem['PositionTicks'])

    EmbyServerPlayback.API.session_progress(PlayingItem)
    xbmc.log("EMBY.hooks.player: -->[ paused ]", 0) # LOGDEBUG

def Resume():
    xbmc.log("EMBY.hooks.player: [ onPlayBackResumed ]", 1) # LOGINFO

    if not EmbyServerPlayback or 'ItemId' not in PlayingItem:
        return

    if utils.WatchTogetherInProgress:
        for SessionId in utils.WatchTogetherParticipants:
            if SessionId != EmbyServerPlayback.EmbySession[0]['Id']:
                EmbyServerPlayback.API.send_unpause(SessionId)

    globals()["PlayingItem"]['IsPaused'] = False
    EmbyServerPlayback.API.session_progress(PlayingItem)
    xbmc.log("EMBY.hooks.player: --<[ paused ]", 0) # LOGDEBUG

def Stop(EventData):
    xbmc.log("EMBY.hooks.player: [ onPlayBackStopped ]", 1) # LOGINFO
    utils.SyncPause['playing'] = False
    EventData = json.loads(EventData)

    if EventData['end']:
        stop_playback(True, False)
    else:
        stop_playback(True, True)

    xbmc.log("EMBY.hooks.player: --<[ playback ]", 1) # LOGINFO

def SETVolume(EventData):
    EventData = json.loads(EventData)
    globals().update({"Muted": EventData["muted"], "Volume": EventData["volume"]})

    if not EmbyServerPlayback or 'ItemId' not in PlayingItem:
        return

    globals()["PlayingItem"].update({'VolumeLevel': Volume, 'IsMuted': Muted})
    EmbyServerPlayback.API.session_progress(PlayingItem)

def stop_playback(delete, Stopped):
    xbmc.log(f"EMBY.hooks.player: [ played info ] {PlayingItem} / {KodiMediaType}", 1) # LOGINFO

    if True: # Stop clients
        for SessionId in utils.WatchTogetherParticipants:
            if SessionId != EmbyServerPlayback.EmbySession[0]['Id']:
                EmbyServerPlayback.API.send_stop(SessionId)

    utils.WatchTogetherInProgress = False
    utils.WatchTogetherParticipants = []

    if MultiselectionDone or not EmbyServerPlayback:
        return

    PlayingItemLocal = PlayingItem.copy()
    globals().update({"PlayBackEnded": True, "PlayingItem": {'CanSeek': True, 'QueueableMediaTypes': "Video,Audio", 'IsPaused': False}})

    # remove cached query for next up node
    if KodiMediaType == "episode" and LibraryId in EmbyServerPlayback.Views.ViewItems:
        CacheId = f"next_episodes_{EmbyServerPlayback.Views.ViewItems[LibraryId][0]}"
        xbmc.log(f"EMBY.hooks.player: [ played info cache Id ] {CacheId}", 1) # LOGINFO

        if CacheId in pluginmenu.QueryCache:
            xbmc.log("EMBY.hooks.player: [ played info clear cache ]", 1) # LOGINFO
            pluginmenu.QueryCache[CacheId][0] = False

    close_SkipIntroDialog()
    close_SkipCreditsDialog()

    # Trailer is playing, skip
    if SkipItem:
        return

    # Trailers for native content
    if not Stopped:
        # Play init item after native content trailer playback
        if TrailerPath and not EmbyServerPlayback.http.Intros:
            globals()["TrailerPath"] = "START_MAIN_CONTENT"
            EmbyServerPlayback.http.Intros = []
            utils.XbmcPlayer.play(playlist, None, False, playlistIndex)
            return

        # play trailers for native content
        if EmbyServerPlayback.http.Intros:
            play_Trailer()
            return

    EmbyServerPlayback.http.Intros = []
    globals()["TrailerPath"] = ""
    globals()["SkipAVChange"] = False

    if 'ItemId' not in PlayingItemLocal:
        return

    # Set watched status
    Runtime = int(PlayingItemLocal['RunTimeTicks'])
    PlayPosition = int(PlayingItemLocal['PositionTicks'])
    EmbyServerPlayback.API.session_stop(PlayingItemLocal)

    if LiveStreamId:
        EmbyServerPlayback.API.close_livestream(LiveStreamId)

    if delete:
        if utils.offerDelete:
            if Runtime > 10:
                if PlayPosition > Runtime * 0.90:  # 90% Progress
                    DeleteMsg = False

                    if KodiMediaType == 'episode' and utils.deleteTV:
                        DeleteMsg = True
                    elif KodiMediaType == 'movie' and utils.deleteMovies:
                        DeleteMsg = True

                    if DeleteMsg:
                        xbmc.log("EMBY.hooks.player: Offer delete option", 1) # LOGINFO

                        if utils.Dialog.yesno(heading=utils.Translate(30091), message=utils.Translate(33015), autoclose=int(utils.autoclose) * 1000):
                            EmbyServerPlayback.API.delete_item(PlayingItemLocal['ItemId'])
                            EmbyServerPlayback.library.removed((PlayingItemLocal['ItemId'],))

    start_workers()

def play_Trailer(): # for native content
    Path = EmbyServerPlayback.http.Intros[0]['Path']
    li = listitem.set_ListItem(EmbyServerPlayback.http.Intros[0], EmbyServerPlayback.ServerData['ServerId'])
    del EmbyServerPlayback.http.Intros[0]

    if Path.startswith('\\\\'):
        Path = Path.replace('\\\\', "smb://", 1).replace('\\\\', "\\").replace('\\', "/")

    globals()["TrailerPath"] = Path
    li.setPath(Path)
    utils.XbmcPlayer.play(Path, li)

def PositionTracker():  # threaded
    LoopCounter = 0
    xbmc.log("EMBY.hooks.player: THREAD: --->[ position tracker ]", 1) # LOGINFO

    while EmbyServerPlayback and "ItemId" in PlayingItem and not utils.SystemShutdown:
        if not utils.sleep(1):
            try:
                Position = int(utils.XbmcPlayer.getTime())
            except:
                break

            xbmc.log(f"EMBY.hooks.player: PositionTracker: Position: {Position} / IntroStartPositionTicks: {IntroStartPositionTicks} / IntroEndPositionTicks: {IntroEndPositionTicks} / CreditsPositionTicks: {CreditsPositionTicks}", 0) # LOGDEBUG

            if utils.enableSkipIntro:
                if IntroEndPositionTicks and IntroStartPositionTicks < Position < IntroEndPositionTicks:
                    if not SkipIntroJumpDone:
                        globals()["SkipIntroJumpDone"] = True

                        if utils.askSkipIntro:
                            if utils.skipintroembuarydesign:
                                SkipIntroDialogEmbuary.show()
                            else:
                                SkipIntroDialog.show()
                        else:
                            jump_Intro()
                            LoopCounter = 0
                            continue
                else:
                    close_SkipIntroDialog()

            if utils.enableSkipCredits:
                if CreditsPositionTicks and Position > CreditsPositionTicks:
                    if not SkipCreditsJumpDone:
                        globals()["SkipCreditsJumpDone"] = True

                        if utils.askSkipCredits:
                            SkipCreditsDialog.show()
                        else:
                            jump_Credits()
                            LoopCounter = 0
                            continue
                else:
                    close_SkipCreditsDialog()

            if LoopCounter % 10 == 0: # modulo 4
                globals()["PlayingItem"]['PositionTicks'] = Position * 10000000
                xbmc.log(f"EMBY.hooks.player: PositionTracker: Report progress {PlayingItem['PositionTicks']}", 0) # LOGDEBUG
                EmbyServerPlayback.API.session_progress(PlayingItem)
                LoopCounter = 0

            LoopCounter += 1

    globals()["PositionTrackerThread"] = False
    xbmc.log("EMBY.hooks.player: THREAD: ---<[ position tracker ]", 1) # LOGINFO

def jump_Intro():
    xbmc.log(f"EMBY.hooks.player: PositionTracker: Skip intro jump {IntroEndPositionTicks}", 1) # LOGINFO
    utils.XbmcPlayer.seekTime(IntroEndPositionTicks)
    globals()["PlayingItem"]['PositionTicks'] = IntroEndPositionTicks * 10000000
    EmbyServerPlayback.API.session_progress(PlayingItem)
    globals()["SkipIntroJumpDone"] = True

def jump_Credits():
    xbmc.log(f"EMBY.hooks.player: PositionTracker: Skip credits jump {CreditsPositionTicks}", 1) # LOGINFO
    utils.XbmcPlayer.seekTime(PlayingItem['RunTimeTicks'] / 10000000)
    globals()["PlayingItem"]['PositionTicks'] = PlayingItem['RunTimeTicks']
    globals()["SkipCreditsJumpDone"] = True

# Continue sync jobs
def start_workers():
    if not utils.sleep(2):
        for _, EmbyServer in list(utils.EmbyServers.items()):
            EmbyServer.library.RunJobs()

def close_SkipIntroDialog():
    if utils.skipintroembuarydesign:
        if SkipIntroDialogEmbuary.dialog_open:
            SkipIntroDialogEmbuary.close()
    else:
        if SkipIntroDialog.dialog_open:
            SkipIntroDialog.close()

def close_SkipCreditsDialog():
    if SkipCreditsDialog.dialog_open:
        SkipCreditsDialog.close()

def queuePlayingItem(EmbyID, MediasourceID, PlaySessionId, IntroStartPosTicks, IntroEndPosTicks, CreditsPosTicks, LiveStreamID=None):  # loaded directly from webservice.py for addon content, or via "onAVStarted" for native content
    xbmc.log("EMBY.hooks.player: [ Queue playing item ]", 1) # LOGINFO
    globals()["LiveStreamId"] = LiveStreamID

    if not utils.syncduringplayback:
        utils.SyncPause['playing'] = True

    if IntroStartPosTicks:
        globals()["IntroStartPositionTicks"] = IntroStartPosTicks
    else:
        globals()["IntroStartPositionTicks"] = 0

    if IntroEndPosTicks:
        globals()["IntroEndPositionTicks"] = IntroEndPosTicks
    else:
        globals()["IntroEndPositionTicks"] = 0

    if CreditsPosTicks:
        globals()["CreditsPositionTicks"] = CreditsPosTicks
    else:
        globals()["CreditsPositionTicks"] = 0

    globals()["QueuedPlayingItem"] = {'CanSeek': True, 'QueueableMediaTypes': "Video,Audio", 'IsPaused': False, 'ItemId': int(EmbyID), 'MediaSourceId': MediasourceID, 'PlaySessionId': PlaySessionId, 'PositionTicks': 0, 'RunTimeTicks': 0, 'VolumeLevel': Volume, 'IsMuted': Muted}

def Cancel():
    xbmc.executeJSONRPC('{"jsonrpc":"2.0","method":"Player.Stop","params":{"playerid":1},"id":1}') # workaround due to Kodi gets unstable by stop
    utils.SyncPause['playing'] = False
    start_workers()

def load_KodiItem(TaskId, KodiItemId, Type, Path):
    videodb = dbio.DBOpenRO("video", TaskId)

    if Type == "movie":
        KodiItem = videodb.get_movie_metadata_for_listitem(KodiItemId, Path)
    elif Type == "episode":
        KodiItem = videodb.get_episode_metadata_for_listitem(KodiItemId, Path)
    elif Type == "musicvideo":
        KodiItem = videodb.get_musicvideos_metadata_for_listitem(KodiItemId, Path)
    else:
        KodiItem = {}

    dbio.DBCloseRO("video", TaskId)

    if KodiItem:
        return listitem.set_ListItem_from_Kodi_database(KodiItem, Path)[1]

    return None

def replace_playlist_listitem(ListItem, PlaySessionId, QueryData, Path):
    globals()["PlaylistRemoveItem"] = playlist.getposition() # old listitem will be removed after play next
    playlist.add(Path, ListItem, PlaylistRemoveItem + 1)
    queuePlayingItem(QueryData['EmbyID'], QueryData['MediasourceID'], PlaySessionId, QueryData['IntroStartPositionTicks'], QueryData['IntroEndPositionTicks'], QueryData['CreditsPositionTicks'])

# Init Dialog here, need from inside class
SkipIntroDialog.set_JumpFunction(jump_Intro)
SkipIntroDialogEmbuary.set_JumpFunction(jump_Intro)
SkipCreditsDialog.set_JumpFunction(jump_Credits)
